#ifndef COLOURLINESPEC_H_
#define COLOURLINESPEC_H_

//#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <gtk/gtk.h>

using namespace std;

/** \brief encapsulates a method to specify colour and line or point plotting
This class tries to encapsulate octave style colours and line type specification.
See ColourLineSpec::ColourLineSpec for a description

Additionally, as this is only a header file, you must put something like the following into your C++ source file :

// setup the possible plot colours ... NOTE: if you want to use the Plot::grid method, you HAVE TO make sure that "a" is mapped to a colour - in this case, gray.
int ColourLineSpec::colourCnt=10; // match this number with the number of colours below
const char *ColourLineSpec::shortColours[]= {(char *)"y",(char *)"m",(char *)"c",(char *)"r",(char *)"g",(char *)"b",(char *)"w",(char *)"k",(char *)"o",(char *)"a"}; // standard colours
const char *ColourLineSpec::X11Colours[]= {(char *)"yellow", (char *)"magenta", (char *)"cyan", (char *)"red", (char *)"green", (char *)"blue", (char *)"white", (char *)"black", (char *)"orange", (char *)"gray"};

This class allows you to specify plot spcifications like so :
"c1" for a cyan plot of line width 1. Or "y2:" for a yellow dotted plot of size 2.
i.e. : plot(x,y,x.size(),"y2");
*/
class ColourLineSpec {
    static int colourCnt; ///< The number of colours defined in the following char ** variables
    static const char *shortColours[]; ///< Default octave colours which match ColourLineSpec::X11Colours
    static const char *X11Colours[]; ///< Default X11 colours to match ColourLineSpec::shortColours
    bool sizeNotSpecified; ///< Set to true if the size was not specified ... i.e. "r" for red with no size
    bool colourNotSpecified; ///< Set to true if the colour was not specified ... i.e. "12" for size 12 with no colour
    int size; ///< The size of the line or point
    GdkColor colour; ///< The colour holder
public:
    bool linePlot; ///< The indication for lines or points plotting

    /** \brief default constructor with blue colour
    Initiates the defaul blue line type of size 1
    */
    ColourLineSpec(void) {
        setColourAndLinesOrPoints("b1");
    }

    /** \brief use a octave like string to define the line/points or colour
    Speciy whether to use points or lines and the colour.
    For example, "r" means, plot red with lines, ":k" means plot black in points.
    Also you can specify size, eg : ":r10" means points, red, size 10
    */
    ColourLineSpec(const char* clsChar) {
        if (strlen(clsChar)==0)
            setColourAndLinesOrPoints("b1");
        else
            setColourAndLinesOrPoints(clsChar);
    }

    /** Enquire whether the size was specified as well as the colour.
    For example:
    \code
    ColourLineSpec cls("r"); // is red without size.
    cls.wasSizeSpecified(); // is false
    \endcode
    \return true if the size was specified, false otherwise
    */
    bool wasSizeSpecified(void){
        return !sizeNotSpecified;
    }

    /** Enquire whether the colour was specified as well as the size
    For example:
    \code
    ColourLineSpec cls("12"); // is size 12 without colour.
    cls.wasColourSpecified(); // is false
    \endcode
    \return true if the colour was specified, false otherwise
    */
    bool wasColourSpecified(void){
        return !colourNotSpecified;
    }

    /** Manually specify the size
    \param s the size
    */
    void setSize(int s){
        sizeNotSpecified=false;
        size=s;
    }

    /** Request the size
    \return the size
    */
    int getSize(void){
        return size;
    }

    /** Manually specify the colour
    \param c the GtkColor
    */
    void setColour(GdkColor c){
        colourNotSpecified=false;
        colour=c;
    }

    /** Manually specify the colour
    \param c the char* color (could be something like "red" or "#b2b2b2")
    */
    void setColour(const char* c){
        colourNotSpecified=false;
        gdk_color_parse(c,&colour);
    }

    /** Request the colour
    \return the GdkColor
    */
    GdkColor getColour(void){
        return colour;
    }

    /** Request the colour string
    \return the GdkColor as a newly allocated string
    */
    gchar *getColourString(void){
        return gdk_color_to_string(&colour);
    }

    /** \brief interprets the octave style line/colour spec.
    By default implements lines specify points using the ':' character.
    Understands the default octave colours.

    */
    void setColourAndLinesOrPoints(const char* clsChar) {
        //colourCnt=sizeof(shortColours)/sizeof(char*);
        //cout<<"colourCnt = "<<colourCnt<<endl;
        sizeNotSpecified=true;
        colourNotSpecified=true;

        linePlot=1;
        string clsStr(clsChar);
        gdk_color_parse("blue",&colour);
        //cout<<"ColourLineSpec::setColourAndLinesOrPoints colour = "<<"blue"<<endl;
        for (int i=0; i<colourCnt; i++)
            if (clsStr.find(shortColours[i])!=string::npos) {
                gdk_color_parse(X11Colours[i],&colour);
                colourNotSpecified=false; // indicate that the colour was specified
                //      cout<<"ColourLineSpec::setColourAndLinesOrPoints colour = "<<X11Colours[i]<<endl;
                break;
            }

        if (clsStr.find(':')!=string::npos) { // check for the points plotting style
            linePlot=false;
            //cout<<"ColourLineSpec::setColourAndLinesOrPoints linePlot = "<<linePlot<<endl;
        } //else            cout<<"ColourLineSpec::setColourAndLinesOrPoints linePlot = "<<linePlot<<endl;

        size=1;
        unsigned int minC=clsStr.find_first_of("0123456789");
        //cout<<"clsStr : "<<clsStr<<endl;
        if ((minC!=string::npos) & (minC<=clsStr.size())) {
            int maxC=clsStr.find_last_of("0123456789");
            size=atoi(clsStr.substr(minC,maxC-minC+1).c_str());
            sizeNotSpecified=false; // indicate that the size was specified
        }
        //cout<<"ColourLineSpec::setColourAndLinesOrPoints size = "<<size<<endl;
    }

    ColourLineSpec &operator=(string &c){
        if (c.size()==0)
            setColourAndLinesOrPoints("b1");
        else
            setColourAndLinesOrPoints(c.c_str());
        return *this;
    }

    ColourLineSpec &operator=(char *c){
        if (strlen(c)==0)
            setColourAndLinesOrPoints("b1");
        else
            setColourAndLinesOrPoints(c);
        return *this;
    }


    static const char ** getColourLabels(void){
        return shortColours;
    }
};

#endif // COLOURLINESPEC_H_
