/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
 */
#ifndef CONTAINER_H_
#define CONTAINER_H_
#include <gtk/gtk.h>
#include "mffm/LinkList.H"
#include "Widget.H"

/** Container

Base class for those which inherit from the Container

*/
class Container : public Widget {
protected:

    /** Should be overloaded by inheriting classes if it has more sophisticated paching techniques, such as Box
    */
    virtual void pack(GtkWidget *obj){
         gtk_container_add(GTK_CONTAINER(widget),obj);
    }

public:
    /*! Method to get the GtkWidget
    \return box The box GtkWidget
    */
    GtkWidget* getWidget(void) {
        return widget;
    }

    /*! Method to show the box.
    \return box the GtkWidget* to the box
    */
    GtkWidget* show(void) {
        gtk_widget_show(widget);
        return widget;
    }
    /*! Method to hide the box.
    \return box the GtkWidget* to the box
    */
    GtkWidget* hide(void) {
        gtk_widget_hide(widget);
	return widget;
    }

    /** Set the border size of the container class
    \param size the size of the border
    */
    void setBorder(unsigned int size){
        gtk_container_set_border_width(GTK_CONTAINER(widget), size);
    }

    /** Get the border size of the container class
    \return the size of the border
    */
    unsigned int getBorder(void){
        return gtk_container_get_border_width(GTK_CONTAINER(widget));
    }

        /*! Operator to pack a different box in this box
    \param b The box to add.
    \return Returns a Container reference for further processing.
    */
    Container& operator <<(Container& b) {
        return operator<<(b.getWidget());
    }

    /*! Operator to pack a pointed to box in this box
    \param b A pointer to the box to add.
    \return Returns a Container reference for further processing.
    */
    Container& operator <<(Container * b) {
        return operator<<(b->getWidget());
    }

    /*! Pack a GtkWidget into the box.
    \param b A pointer to the GtkWidget to pack.
    \return Returns a Container reference for further processing.
    */
    virtual Container& operator <<(GtkWidget * b) {
        pack(b);
        return *this;
    }

    /*! Pack a GtkWidget into the box.
    \param ll A LinkList of GtkWidget* for packing from the first to the last
    \return Returns a Container reference for further processing.
    */
    Container& operator <<(LinkList<GtkWidget *> &ll) {
        ll.grab(1);// load using next from the first
        for (int i=1;i<=ll.getCount();i++,ll.next())
            operator<<(ll.current());
        return *this;
    }

    /*! Unpack a widget from the box.
        Note that removal of the widget can destroy it if the only reference is the box.
    \param b The widget to be unpacked from the box.
    \return Returns a Container reference for further processing.
    */
    Container& operator >>(GtkWidget * b) {
        gtk_container_remove(GTK_CONTAINER(widget), b);
        return *this;
    }

    /*! Unpack a box pointer from the box.
        Note that removal of the widget can destroy it if the only reference is the box.
    \param b The pointer to the box to be unpacked from the box.
    \return Returns a Container reference for further processing.
    */
    Container& operator >>(Container * b) {
        gtk_container_remove(GTK_CONTAINER(widget), b->getWidget());
        return *this;
    }

    /*! Unpack a box from the box.
        Note that removal of the widget can destroy it if the only reference is the box.
    \param b The pointer to the box to be unpacked from the box.
    \return Returns a Container reference for further processing.
    */
    Container& operator >>(Container &b) {
        gtk_container_remove(GTK_CONTAINER(widget), b.getWidget());
        return *this;
    }

//    Container& operator=(const Container b){
//        widget=b.getWidget();
//    }
};
#endif // CONTAINER_H_
