/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
*/
#ifndef OCTAVE_H_
#define OCTAVE_H_

//#ifndef OCTAVE_ERROR_OFFSET
//#define OCTAVE_ERROR_OFFSET -2000
//#endif
//#define GEN_ERROR_OCTAVE(errNo) OCTAVE_ERROR_OFFSET+errNo

//#define OCTAVE_ERROR_INCORRECT_RETCOUNT GEN_ERROR_OCTAVE(-1)
//#define OCTAVE_ERROR_INCORRECT_RETCOUNT_STR string("Octave returned the incorrect number of elements - error")

#include <Eigen/Dense>
#include <string>
#include <vector>
using namespace std;

/** Class to interface to GNU Octave (octave.org)
requires octave headers installed on the system

First implementation is limited to matrix inputs and outputs to arbitrary .m files.

Usage like so :
\code
    // start octave
    vector<string> args(3); args[0]=string("--silent"); args[1]=string("--path"); args[2]=string("/tmp");
    Octave octave(args);

    // create the input and output vectors. Load the input vector
    vector<Eigen::Matrix<float, Dynamic, Dynamic> > input(1), output;
    Eigen::Matrix<float, Dynamic, Dynamic> a(10,10); Eigen::Matrix<float, Dynamic, Dynamic>::Zero(10,10);
    input[0]=a;

    // run the file yourMFile.m with the input arguments
    output = octave.runM("yourMFile", input); // where yourMFile.m is on the path say /tmp/yourMFile.m

    // output now holds the results

\endcode

The Octave.C file must be compiled like so :
g++ `mkoctfile -p CFLAGS` `mkoctfile -p INCFLAGS` `pkg-config --cflags eigen3` -fPIC -c Octave.C

Alternatively, if you are using autotools, do the following (note if you link -lfftw3f elsewhere, you may need to comment out the -p FFTW_LIBS line to stop conflicts) :
\code
lib_LTLIBRARIES = libGTKIOStream.la
libGTKIOStream_la_SOURCES = gtkiostream/Octave.C
libGTKIOStream_la_CPPFLAGS = $(EIGEN_CFLAGS) $(MKOCTFILE_CFLAGS)

Where the following are in your configure.ac :
# eigen3
PKG_CHECK_MODULES(EIGEN, eigen3 >= 3.0.0)
AC_SUBST(EIGEN_CFLAGS)
AC_SUBST(EIGEN_LIBS)

# user supplied mkoctfile program
AC_ARG_WITH([mkoctfile],
  [  --with-mkoctfile=PATHTOMKOCTFILE  The 'mkoctfile' program.],
  [mkoctfile="$withval"],[mkoctfile="no"])
# get mkoctfile program
if test "$mkoctfile" == "no"; then
  AC_PATH_PROG([mkoctfile],[mkoctfile],[no],[$prefix/bin:$PATH])
fi
test $mkoctfile = "no" && AC_MSG_ERROR([mkoctfile not found])
AC_SUBST([MKOCTFILE],[$mkoctfile])
# get mkoctfile compiler flags
CAT="cat"
MKOCTFILE_CFLAGS="$(echo $($MKOCTFILE -p INCFLAGS | $CAT) | sed -re 's|\\|/|g')"
MKOCTFILE_LIBS="$(echo \
  $(unset RLD_FLAG; $MKOCTFILE -p RLD_FLAG | $CAT) \
  $(unset LFLAGS; $MKOCTFILE -p LFLAGS | $CAT) \
  $(unset LIBOCTINTERP; $MKOCTFILE -p LIBOCTINTERP | $CAT) \
  $(unset LIBOCTAVE; $MKOCTFILE -p LIBOCTAVE | $CAT) \
  $(unset LIBCRUFT; $MKOCTFILE -p LIBCRUFT | $CAT) \
  $(unset FFTW_LIBS; $MKOCTFILE -p FFTW_LIBS | $CAT) \
  $(unset BLAS_LIBS; $MKOCTFILE -p BLAS_LIBS | $CAT) \
  $(unset FLIBS; $MKOCTFILE -p FLIBS | $CAT) \
  $(unset LIBS; $MKOCTFILE -p LIBS | $CAT) \
  | sed -re 's|\\|/|g')"
AC_SUBST([MKOCTFILE_CFLAGS])
AC_SUBST([MKOCTFILE_LIBS])

Finally, to link your program :
INCLUDES                = $(FFTW_CFLAGS) $(EIGEN_CFLAGS) $(GTKDATABOX_CFLAGS) -I$(top_srcdir)/test/gtkiostream
LDADD                   = $(EIGEN_LIBS) $(GTKDATABOX_LIBS) libGTKIOStream.la

\endcode

*/
class Octave {
public:
    /** Constructor starts an octave instance using args to pass to octave
    \param args Arguments to pass into octave
    */
    Octave(const vector<string> &args);

    /// Destructor
    virtual ~Octave();

    /**
    \tparam TYPE The Eigen::Matrix types to input and return
    \param commandName The .m file name to run
    \param in The vector of Eigen::Matrix (the vector of matrices) to input to the .m file.
    */
    template<class TYPE>
    vector<Eigen::Matrix<TYPE, Eigen::Dynamic, Eigen::Dynamic> > runM(const char* commandName, vector<Eigen::Matrix<TYPE, Eigen::Dynamic, Eigen::Dynamic> > &in);
};
#endif // OCTAVE_H_
