/* Copyright 2000-2013 Matt Flax <flatmax@flatmax.org>
   This file is part of GTK+ IOStream class set

   GTK+ IOStream is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   GTK+ IOStream is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You have received a copy of the GNU General Public License
   along with GTK+ IOStream
 */
#ifndef THREAD_H_
#define THREAD_H_

#ifdef USE_USE_GLIB_THREADS
#include <glib.h>
#else
#include <pthread.h>
#include <errno.h>
#include <stdio.h>
#endif

#include <iostream>
using namespace std;

/** Class to spawn a thread and meet an exited thread.
*/
class Thread {

#ifdef USE_GLIB_THREADS
    GThread* thread; ///< The thread pointer
    GError *threadErrorPtr; ///< The thread error pointer
#else
    pthread_t thread; ///< The thread structure
#endif
public:
    /** Constructor
    */
    Thread(void) {
#ifdef USE_GLIB_THREADS
        thread=NULL;
        threadErrorPtr=NULL;
#endif
    }

    /** Destructor
    */
    virtual ~Thread(void) {
#ifdef USE_GLIB_THREADS
        g_thread_join(thread);
        thread=NULL;
#else
//         void *retVal;
         pthread_cancel(thread); // this returns error of ESRCH if the thread is already finished

//        int threadResp=pthread_join(thread, &retVal);
        // on destruction, not interested in the return value here, just want to make sure the thread has exited.
//        if (threadResp!=0) {
//            errno=threadResp;
//            perror("Thread::meetThread : Couldn't meet the thread");
//            cerr<<"Most likely that the thread isn't running."<<endl;
//        }
#endif
    }

    /** Start a thread running start_routine, passing data to that function
    \param start_routine The function to run after the thread has been started
    \param data The argument to be pased to the thread
    \param zero on success, a different number on failure
    */
#ifdef USE_GLIB_THREADS
    int run(void (*start_routine) (void *), void *data) {
#else
    int run(void *(*start_routine) (void *), void *data) {
#endif
#ifdef USE_GLIB_THREADS
        thread = g_thread_create((GThreadFunc)start_routine, (gpointer) data, TRUE, &threadErrorPtr);
        // TODO : check the response of g_thread_create on error
        return (int)thread;
#else
        pthread_attr_t attr;
        pthread_attr_init(&attr);
        pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
//    int pthread_setcanceltype(int type, int *oldtype);

        int threadResp=pthread_create(&thread, &attr, start_routine, data);
        if (threadResp!=0) {
            errno=threadResp;
            perror("Thread::run : Thread creation failed.");
        }
        pthread_attr_destroy(&attr);
        return threadResp;
#endif
    }

    /** Wait for the thread to end
    \return If using USE_GLIB_THREADS then NULL, if using pthreads, then the argument passed to Thread::exit when called by the exiting thread.
    */
    void *meetThread(void) {
#ifdef USE_GLIB_THREADS
        g_thread_join(thread);
        thread=NULL;
        return NULL;
#else
        void *retVal;
        int threadResp=pthread_join(thread, &retVal);
        if (threadResp!=0) {
            errno=threadResp;
            perror("Thread::meetThread : Couldn't meet the thread");
        }
        return retVal;
#endif
    }

    /** Called to exit the thread passes retVal to the one who calls meetThread
    \param retVal The pointer to pass to the meetThread caller
    */
    void exit(void *retVal) {
#ifndef USE_GLIB_THREADS
        pthread_exit(retVal);
#endif
    }
};
#endif // THREAD_H_
